一个 Linux 驱动的微波炉
================================================================================

[linux.conf.au](http://linux.conf.au/)里的人们都有一种想到什么就动手去实现的想法。随着硬件开源运动不断地发展壮大，这种想法越来越多，也与现实世界联系的越来越紧密，而不仅仅存在于数字世界中。David Tulloh用他制作的[Linux驱动的微波炉 [WebM]](http://mirror.linux.org.au/linux.conf.au/2016/04_Thursday/D4.303_Costa_Theatre/Linux_driven_microwave.webm)来展示一个差劲的微波炉会多么难用，以及说明他的项目可以改造这些微波炉使得它们不那么讨人厌。

Tulloh的故事要从他买到了一个公认很便宜的微波炉开始说起，它的用户界面比其它微波炉默认的还要糟糕。设定时间时必须使劲按按钮以至于把微波炉都向后推了一段距离——而事实上必须要用力拉仓门把手才能把微波炉拖回原来的位置，这形成了一个“优雅”的平衡。当然这只是极端情况。Tulloh郁闷的是因为这个微波炉近十年来都没有一丁点明显的改善。他可能买到了一个又小又便宜的微波炉，而且特点是大部分人不研究使用手册就不会使用它——和智能手机的对比更加明显：智能手机只需知道一点点的操作指南并且被广泛使用。

改造这个微波炉不一定没有前途，“让微波炉重获新生”——这个想法成为了一个原型，如果Tulloh可以再平衡一下想做的功能和需求之间的关系的话他希望这变成一个众筹项目：一个Linux驱动的微波炉。

![](https://static.lwn.net/images/2016/lca-oven-sm.jpg)

## 加一点新奇的小玩意

如果把“Linux”和“微波炉”联系在一起的话，你可能会想到给微波炉加上一个智能手机式的触摸屏和网络链接，然后再通过社区做一款微波炉的“革命性”的手机应用，想到这些就像做菜想到分享食谱一样显而易见。但Tulloh的目标和他的原型远远超过这些，他做了两个新奇的功能——热感相机和称量物体质量的称重装置。

这个热感相机提供一个可以精确到两度的八乘八像素的图像，这足够发现一杯牛奶是否加热到沸腾或者牛排是否解冻到快不能用来烹饪。不论发生哪种情况，都可以减小功率或者关掉它。而且在必要的时候会发出警报。这可能不是第一个可以检测温度的微波炉——GE在十年前就开始卖带温度探针的微波炉了——但是一个一直工作的内置传感器比一个手工探针有用多了，尤其是有一个可用的API支持的时候。

第二个新发明是一个嵌入的称重装置，它可以在加热之前称量食物(和容器)。很多食谱根据质量大小给出指导的烹饪时间，很多微波炉支持你手动输入质量以便它帮你计算。利用内置的称重装置，这一过程可以变成自动化的。在许多微波炉的转盘下面稳固地放置一个称重装置是一个机械方面的挑战，不过Tulloh觉得这个问题不难处理。相反，他对微波炉的设计是基于“平板”或者“平板挂车”的风格——在四角各放置一个传感器，这不仅在机械实现上很简单而且很好的达到了要求。

一旦你有了这些额外添加的并与逻辑引擎相连的质量温度传感器，你可以去尝试更多好玩的可能。一杯刚从冰箱里拿出来的冰牛奶的质量温度分布可能会有适度误差。Tulloh发现可以监检测到这种情况，而且提供一些有关的像“煮沸”或者“加热”的选项也是容易做到的（下面有一个模拟的界面，可点击操作的版本请点击右边链接 [here](http://mwgui.tulloh.id.au/)）

![](https://static.lwn.net/images/2016/lca-ovengui-sm.png)

## 改造陈旧的东西

除了才开发出来的新功能，Tulloh还想要提升那些原本就提供的功能。可能不是所有微波炉的门把手都像Tulloh那个廉价的一样僵硬，但是很少有微波炉将把手设计的让残疾人也能轻松使用。这些缺陷都是可调整的，尤其是在美国，微波炉应该在仓门关闭的时候给出一个确定关闭的提示。这种确认必须是可靠的以预防那些伪劣产品，所以在仓门闭合时固定的槽位里添加一个短杆以确认仓门开闭状态，不至于误使微波炉在仓门开着的时候工作。事实上，必须要有两个相互联系的机关，如果他们提供的结果不一致，保险丝必须断开以便启动一个呼叫服务。Tulloh认为提供一个磁力门闩有更大的灵活性（包含简单的软件控制）并且像磁控也同样用于[磁性钥匙锁](https://en.wikipedia.org/wiki/Magnetic_keyed_lock)，它可以让磁力门闩确认微波炉门是否关闭。

微波炉的另一个痛点是它会发出令人厌烦的声音。Tulloh去掉了蜂鸣器并且使用香蕉派（类似于树莓派的单片机开发板）控制他的微波炉。这可以通过一个把文本转换成语音的系统来用令人愉悦而且可配置的警报来提示和引导使用者。显然，下一步就是装上一个用来控制声音的扩音器。

许多微波炉除了定时和设置功率档位之外还可以做更多的事情——它们为烹饪，加热，化冻等提供一系列的功率档位。加上一个精确的温度测量装置感觉会为这个图表大大扩展这个列表。Andrew Tridgell对一个问题很好奇，加热巧克力——一个需要非常精确的温度控制的过程——是否是可能的。Tulloh没有过这方面的经验，他不敢保证这个一定可以，但是这个实验结果的确值得期待。即使没做成这件事，它也显出了潜在价值——社区接下来可以更进一步去做这件事。

## 实用性怎么样？

Tulloh十分乐意向全世界分享这个linux驱动的微波炉，他希望看到（因为这件事）形成一个社区并且想看到它接下来的走势。买一个现成的微波炉并且替换掉里面的电子元件看起来不是一个可行的点子。最后的结果可能会很糟，而买一个小巧智能的微波炉必然要花掉（比自己改造）更多的钱，但是潜在的顾客不想在他们的厨房里看到乱七八糟又不协调的东西。

许多零件都是现成的可以买到的（磁电管，处理器板，热传感器等等），像USB接口的热传感器，而且都很容易安装。软件原型当然也开源在[GitHub](https://github.com/lod?tab=repositories)。微波炉的舱室和门有不小的挑战性并且很可能要定制。Tulloh想要通过提供左侧开仓门的微波炉和颜色多样化的选项来转逆境为机遇。

一个对读者的快速调查：很少有人会贸然承诺他会为了一个全新的升级过的烤箱付出1000澳大利亚元。当然，很难知道是否会有充足的时间和足够多的读者来完成这个调查。这整个项目看起来很有趣。所以Tulloh的[博客](http://david.tulloh.id.au/category/microwave/) (点击这里)也很值得一看。

------------------------------------------------------------------------------

via: https://lwn.net/Articles/674877/

作者：Neil Brown
译者：[yuba0604](https://github.com/yuba0604)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创翻译，[Linux中国](https://linux.cn/) 荣誉推出
